"""
Module that contains the command line application.

Why does this file exist, and why not put this in __main__?

You might be tempted to import things from __main__ later,
but that will cause problems: the code will get executed twice:

- When you run `python -m mvodb` python will execute
  ``__main__.py`` as a script. That means there won't be any
  ``mvodb.__main__`` in ``sys.modules``.
- When you import __main__ it will get executed again (as a module) because
  there's no ``mvodb.__main__`` in ``sys.modules``.

Also see http://click.pocoo.org/5/setuptools/#setuptools-integration.
"""

import argparse
import os
from functools import lru_cache
from pathlib import Path

import tmdbsimple as tmdb
from guessit import guessit
from langdetect import detect
from langdetect.lang_detect_exception import LangDetectException

tmdb.API_KEY = os.environ.get("TMDB_API_KEY")


LANG = {"English": "eng", "French": "fre"}


class Guess:
    def __init__(self, name):
        self.data = guessit(name)
        self.data["filename"] = name
        self.data["ext"] = name.split(".")[-1]

    def __hash__(self):
        if self.is_movie:
            return hash(("movie", self.data["title"], self.data["year"]))
        elif self.is_episode:
            return hash(("movie", self.data["title"], self.data["season"], self.data["episode"]))
        return hash("unknown")

    def __eq__(self, other):
        return hash(self) == hash(other)

    # def __eq__(self, other):
    #     if self.type != other.type:
    #         return False
    #     if self.is_movie:
    #         return self.title == other.title and self.release_date == other.release_date
    #     elif self.is_episode:
    #         return self.title == other.title and self.season == other.season and self.episode == other.episode
    #     return self.data == other.data

    @property
    def is_episode(self):
        return self.data["type"] == "episode"

    @property
    def is_movie(self):
        return self.data["type"] == "movie"

    def get_new_path(self):
        if self.data["ext"] == "srt":
            try:
                self.data["lang"] = self.data["subtitle_language"].alpha3
            except (KeyError, AttributeError):
                with open(self.data["filename"]) as stream:
                    text = stream.read()
                if text:
                    try:
                        self.data["lang"] = detect(text)
                    except LangDetectException:
                        pass
        if self.is_episode:
            self.data.update(get_info_episode(self.data["title"], self.data["season"], self.data["episode"]))
            return episode_to_path(self.data)
        elif self.is_movie:
            self.data.update(get_info_movie(self.data["title"]))
            return movie_to_path(self.data)
        raise ValueError


@lru_cache()
def get_info_episode(title, season_number, episode_number):
    search = tmdb.Search()
    search.tv(query=title)
    tv_show = search.results[0]
    episode = tmdb.TV_Episodes(tv_show["id"], season_number, episode_number)
    episode.info()
    return {
        "tvshow": tv_show["name"],
        "season": season_number,
        "episode": episode_number,
        "title": episode.name,
    }


@lru_cache()
def get_info_movie(title):
    search = tmdb.Search()
    search.movie(query=title)
    movie = search.results[0]
    return {
        "title": movie["title"],
        "year": movie["release_date"].split("-")[0],
    }


def episode_to_path(data):
    n = data["tvshow"]
    s = f"{data['season']:02}"
    e = f"{data['episode']:02}"
    t = data["title"]
    x = data["ext"]
    if "lang" in data and data["lang"]:
        x = f"{data['lang']}.{x}"
    return f"series/{n}/Season {s}/{n} - S{s}E{e} - {t}.{x}"


def movie_to_path(data):
    t = data["title"]
    y = data["year"]
    x = data["ext"]
    if "lang" in data and data["lang"]:
        x = f"{data['lang']}.{x}"
    return f"movies/{t} ({y})/{t} ({y}).{x}"


def filter_ext(files, whitelist):
    return [f for f in files if os.path.splitext(f)[1][1:].lower() in whitelist]


def get_parser():
    parser = argparse.ArgumentParser(prog="mvodb")
    parser.add_argument("files", nargs="+", metavar="FILE", help="Files to move/rename.")
    parser.add_argument(
        "-y", "--no-confirm", action="store_true", default=False, dest="no_confirm", help="Do not ask confirmation."
    )
    return parser


def main(args=None):
    """The main function, which is executed when you type ``mvodb`` or ``python -m mvodb``."""
    parser = get_parser()
    args = parser.parse_args(args=args)

    buffer = []
    for path in args.files:
        path = Path(path)
        if path.is_dir():
            dir_files = list(path.glob("**/*"))
        else:
            dir_files = [path]

        for file in filter_ext(dir_files, ["srt", "mkv", "mp4", "avi"]):
            guess = Guess(str(file))
            new_path = "/media/mybookplex/multimedia/" + guess.get_new_path()
            buffer.append({"original": file, "new": new_path, "answer": True})

    if not args.no_confirm:
        for item in buffer:
            original, new = item["original"], item["new"]
            answer = input(f"mv '{original}' '{new}' [Yn] ")  # nosec
            if answer not in ("", "y", "Y"):
                item["answer"] = False

    for item in [i for i in buffer if i["answer"]]:
        Path(item["new"]).parent.mkdir(parents=True, exist_ok=True)
        os.rename(item["original"], item["new"])

    return 0



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['mvodb']

package_data = \
{'': ['*']}

install_requires = \
['guessit>=3.1.0,<4.0.0',
 'langdetect>=1.0.7,<2.0.0',
 'tmdbsimple>=2.2.0,<3.0.0']

entry_points = \
{'console_scripts': ['mvodb = mvodb.cli:main']}

setup_kwargs = {
    'name': 'mvodb',
    'version': '0.1.0',
    'description': 'Rename and move files using metadata from online databases.',
    'long_description': '<!--\nIMPORTANT:\n  This file is generated from the template at \'scripts/templates/README.md\'.\n  Please update the template instead of this file.\n-->\n\n# mvodb\n[![pipeline status](https://gitlab.com/pawamoy/mvodb/badges/master/pipeline.svg)](https://gitlab.com/pawamoy/mvodb/pipelines)\n[![coverage report](https://gitlab.com/pawamoy/mvodb/badges/master/coverage.svg)](https://gitlab.com/pawamoy/mvodb/commits/master)\n[![documentation](https://img.shields.io/readthedocs/mvodb.svg?style=flat)](https://mvodb.readthedocs.io/en/latest/index.html)\n[![pypi version](https://img.shields.io/pypi/v/mvodb.svg)](https://pypi.org/project/mvodb/)\n\nRename and move files using metadata from online databases.\n\n## Requirements\nmvodb requires Python 3.6 or above.\n\n<details>\n<summary>To install Python 3.6, I recommend using <a href="https://github.com/pyenv/pyenv"><code>pyenv</code></a>.</summary>\n\n```bash\n# install pyenv\ngit clone https://github.com/pyenv/pyenv ~/.pyenv\n\n# setup pyenv (you should also put these three lines in .bashrc or similar)\nexport PATH="${HOME}/.pyenv/bin:${PATH}"\nexport PYENV_ROOT="${HOME}/.pyenv"\neval "$(pyenv init -)"\n\n# install Python 3.6\npyenv install 3.6.8\n\n# make it available globally\npyenv global system 3.6.8\n```\n</details>\n\n## Installation\nWith `pip`:\n```bash\npython3.6 -m pip install mvodb\n```\n\nWith [`pipx`](https://github.com/cs01/pipx):\n```bash\npython3.6 -m pip install --user pipx\n\npipx install --python python3.6 mvodb\n```\n\n## Usage (as a library)\nTODO\n\n## Usage (command-line)\n```\nusage: mvodb [-h] [-y] FILE [FILE ...]\n\npositional arguments:\n  FILE              Files to move/rename.\n\noptional arguments:\n  -h, --help        show this help message and exit\n  -y, --no-confirm  Do not ask confirmation.\n\n```\n\n\n',
    'author': 'Timothée Mazzucotelli',
    'author_email': 'pawamoy@pm.me',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/pawamoy/mvodb',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



"""
mvodb package.

Rename and move files using metadata from online databases.

If you read this message, you probably want to learn about the library and not the command-line tool:
please refer to the README.md included in this package to get the link to the official documentation.
"""

__all__ = []
